#ifndef VISUALIZATION_TOOLS_H
#define VISUALIZATION_TOOLS_H

#include "types.h"
#include "cloud_segmentation.h"

#include <iostream>
#include <fstream>
#include <vector>
#include <fstream>
#include <string>
#include <CGAL/Polyhedron_3.h>
#include <CGAL/Simple_cartesian.h>
#include <CGAL/IO/Polyhedron_iostream.h>
#include <CGAL/IO/print_wavefront.h>
#include <boost/filesystem.hpp>
#include <CGAL/Polyhedron_incremental_builder_3.h>
#include <CGAL/Polygon_mesh_processing/bbox.h>

#include <cmath>
#include <math.h>
#include <map>

using std::vector;
#pragma warning(default:4244 4103 4267 4396)


namespace RegionGrowing {

#define RAD2DEG		180.0/M_PI
#define DEG2RAD		M_PI/180.0


typedef CGAL::Polyhedron_3<Kernel>              Polyhedron;
typedef Kernel::Point_2 Point_2;


#define PARAM_OFSTREAM_TEXTE std::ios::out
#define OFSTREAM_TEXTE(nomvar, nomfic) std::ofstream nomvar (nomfic, PARAM_OFSTREAM_TEXTE) ; nomvar.setf(std::ios::fixed);


template <class HDS>
class Build_sphere : public CGAL::Modifier_base<HDS> {
private:
	double msize;
	Point_d mcenter;
	bool mpureTriangle;
	double mstep;
public:
    Build_sphere(Point_d center,double size, bool pureTriangle, double step) {msize = size;mcenter=center;mpureTriangle = pureTriangle; mstep=step;}
    void operator()( HDS& hds) {
        // Postcondition: `hds' is a valid polyhedral surface.
        CGAL::Polyhedron_incremental_builder_3<HDS> B(hds, true);

		//const double step = 60;//works for 30, 45, 90

		int ratiostep = 360/mstep;
		int nb_point = ratiostep*(ratiostep/2-1)+2;		//+2 because of the top and bottom points
		int nb_facet = ratiostep*(ratiostep/2);			//nb facet mixed triangle and quad
		if (mpureTriangle)
			nb_facet=(nb_facet-2*ratiostep)*2 + 2*ratiostep;

		B.begin_surface( nb_point, nb_facet);			//nb point, nb facet
        typedef typename HDS::Vertex   Vertex;
        typedef typename Vertex::Point Point;
        B.add_vertex( Point( mcenter.x(), mcenter.y(), msize+mcenter.z()));//phi = 90
		double cosphi, sinphi, costheta,sintheta;

		//add point
		double phi;
		int count = 0;
		for (double theta = 0;theta<=360-mstep;theta+=mstep)
		{
			
			sintheta = sin(theta*DEG2RAD);costheta = cos(theta*DEG2RAD);
			for (double tp_phi = mstep;tp_phi<=180-mstep;tp_phi+=mstep)//skip phi = 90 and phi = -90
			{
				phi = 90 - tp_phi;count++;
				cosphi = cos(phi*DEG2RAD);sinphi = sin(phi*DEG2RAD);
				B.add_vertex( Point( msize*cosphi*costheta+mcenter.x(), msize*cosphi*sintheta+mcenter.y(), msize*sinphi+mcenter.z()));
			}
		}


		B.add_vertex( Point( mcenter.x(), mcenter.y(), -msize+mcenter.z()));//phi = -90
		
		//add facet (top)
		double step_index = 180/mstep-1;//= 5
		for (int count = 1;count<=360/mstep; count++)
		{
        B.begin_facet();
			B.add_vertex_to_facet( 0);
			B.add_vertex_to_facet( (count-1)*step_index+1);

			if (count==360/mstep)
				B.add_vertex_to_facet(1);
			else
				B.add_vertex_to_facet(count*step_index+1);
        B.end_facet();
		}


		if (mpureTriangle)
		{
			for (int start = 1;start <= step_index-1;start++)
			{
				for (int count = 1;count<=360/mstep; count++)
				{
				B.begin_facet();
					B.add_vertex_to_facet( start+(count-1)*step_index);
					B.add_vertex_to_facet( start+(count-1)*step_index+1);
					if (count==360/mstep)
						B.add_vertex_to_facet( start+1 );
					else
						B.add_vertex_to_facet( start+count*step_index+1 );
				B.end_facet();

				B.begin_facet();
					B.add_vertex_to_facet( start+(count-1)*step_index);
					if (count==360/mstep)
					{
						B.add_vertex_to_facet( start+1 );
						B.add_vertex_to_facet( start);
					}
					else
					{
						B.add_vertex_to_facet( start+count*step_index+1 );
						B.add_vertex_to_facet( start+count*step_index);
					}
				B.end_facet();
				}
			}

		}else{

			for (int start = 1;start <= step_index-1;start++)
			{
				for (int count = 1;count<=360/mstep; count++)
				{
				B.begin_facet();
					B.add_vertex_to_facet( start+(count-1)*step_index);
					B.add_vertex_to_facet( start+(count-1)*step_index+1);
					if (count==360/mstep)
					{
						B.add_vertex_to_facet( start+1 );
						B.add_vertex_to_facet( start);
					}
					else
					{
						B.add_vertex_to_facet( start+count*step_index+1 );
						B.add_vertex_to_facet( start+count*step_index);
					}
				B.end_facet();
				}
			}
		}

			
		//add facet (bottom)
		for (int count = 1;count<=360/mstep; count++)
		{
        B.begin_facet();
			B.add_vertex_to_facet( count*step_index);
			B.add_vertex_to_facet( nb_point-1);

			if (count==360/mstep)
				B.add_vertex_to_facet(step_index);
			else
				B.add_vertex_to_facet((count+1)*step_index);

        B.end_facet();
		}

	
		

        B.end_surface();
    }
};



template <class HDS>
class Build_cylinder : public CGAL::Modifier_base<HDS> {
private:
	double msize;
	Point_d mpt1;
	Point_d mpt2;
	bool mpureTriangle;
	double mstep; //works for  30, 45,60, 90 (in degree)
public:
    Build_cylinder(Point_d pt1,Point_d pt2, double size, bool pureTriangle, double step) {msize = size; mpt1=pt1;  mpt2=pt2; mpureTriangle = pureTriangle; mstep=step;}
    void operator()( HDS& hds) {
        // Postcondition: `hds' is a valid polyhedral surface.
        CGAL::Polyhedron_incremental_builder_3<HDS> B(hds, true);

		//const double step = step_deg;//works for 30, 45, 90

		int ratiostep = 360/mstep;
		int nb_point = 2*ratiostep;		//+2 because of the top and bottom points
		int nb_facet = 2*ratiostep;			//nb facet mixed triangle and quad
		//if (mpureTriangle) nb_facet=(nb_facet-2*ratiostep)*2 + 2*ratiostep;

		B.begin_surface( nb_point, nb_facet);			//nb point, nb facet
       // typedef typename HDS::Vertex   Vertex;

		Line lin(mpt1,mpt2); 
		Plane_3 plane1(mpt1,lin.direction()); 
		Plane_3 plane2(mpt2,lin.direction()); 

		//Point_2 ptc=plane1.to_2d(mpt1); 
		std::vector<Point_d> list_pt1;
		std::vector<Point_d> list_pt2;

		for(int i=0;i<ratiostep;i++){
			Point_2 temp_2(cos(i*M_PI*mstep/180),sin(i*M_PI*mstep/180)); 
			Vector_3 B1=plane1.base1(); FT norme1=sqrt(pow(B1.x(),2)+pow(B1.y(),2)+pow(B1.z(),2)); Vector_3 b1=(1/norme1)*B1;		
			Vector_3 B2=plane1.base2(); FT norme2=sqrt(pow(B2.x(),2)+pow(B2.y(),2)+pow(B2.z(),2)); Vector_3 b2=(1/norme2)*B2;	
			Point_d temp_3(mpt1.x()+msize*(b1.x()*temp_2.x()+b2.x()*temp_2.y()),mpt1.y()+msize*(b1.y()*temp_2.x()+b2.y()*temp_2.y()),mpt1.z()+msize*(b1.z()*temp_2.x()+b2.z()*temp_2.y())); 

			list_pt1.push_back(temp_3);
			list_pt2.push_back(plane2.projection(temp_3));

		}


		//add point
		for(int i=0;i<ratiostep;i++) B.add_vertex(list_pt1[i]); //std::cout<<list_pt1[i]<<std::endl;}
		for(int i=0;i<ratiostep;i++) B.add_vertex(list_pt2[i]); //std::cout<<list_pt2[i]<<std::endl;}
		
		//add facet 
		for(int i=0;i<ratiostep-1;i++){
			 B.begin_facet();
			 B.add_vertex_to_facet(i);
			 B.add_vertex_to_facet(i+1);
			 B.add_vertex_to_facet(i+ratiostep);
			 B.end_facet();

			 B.begin_facet();
			 B.add_vertex_to_facet(i+1);
			 B.add_vertex_to_facet(i+ratiostep+1);
			 B.add_vertex_to_facet(i+ratiostep);
			 B.end_facet();
		}
			 
			 B.begin_facet();
			 B.add_vertex_to_facet(ratiostep-1);
			 B.add_vertex_to_facet(0);
			 B.add_vertex_to_facet(2*ratiostep-1);
			 B.end_facet();

			 B.begin_facet();
			 B.add_vertex_to_facet(0);
			 B.add_vertex_to_facet(ratiostep);
			 B.add_vertex_to_facet(2*ratiostep-1);
			 B.end_facet();


        B.end_surface();
    }
};





template <class HDS>
class Build_square : public CGAL::Modifier_base<HDS> {
private:
	std::vector < Point_d > mvpt;
	bool mpureTriangle;
public:
	Build_square(std::vector < Point_d > vpt, bool pureTriangle) { mvpt=vpt; mpureTriangle = pureTriangle;}
    void operator()( HDS& hds) {
        // Postcondition: `hds' is a valid polyhedral surface.
        CGAL::Polyhedron_incremental_builder_3<HDS> B(hds, true);

	

		B.begin_surface(4,2);			//nb point, nb facet
  

		//add point
		for(int i=0;i<4;i++) B.add_vertex(mvpt[i]); 
		
		
		//add facet 
		
			 B.begin_facet();
			 B.add_vertex_to_facet(0);
			 B.add_vertex_to_facet(1);
			 B.add_vertex_to_facet(2);
			 B.end_facet();

			 B.begin_facet();
			 B.add_vertex_to_facet(2);
			 B.add_vertex_to_facet(3);
			 B.add_vertex_to_facet(0);
			 B.end_facet();
		
        B.end_surface();
    }
};



template <class HDS>
class Build_cube : public CGAL::Modifier_base<HDS> {
private:
	std::vector < Point_d > mvpt;
	bool mpureTriangle;
public:
	Build_cube(std::vector < Point_d > vpt, bool pureTriangle) { mvpt=vpt; mpureTriangle = pureTriangle;}
    void operator()( HDS& hds) {
        // Postcondition: `hds' is a valid polyhedral surface.
        CGAL::Polyhedron_incremental_builder_3<HDS> B(hds, true);

	

		B.begin_surface(8,12);			//nb point, nb facet
  

		//add point
		for(int i=0;i<8;i++) B.add_vertex(mvpt[i]); 
		
		
		//add facet 
		
			/* B.begin_facet();
			 B.add_vertex_to_facet(3);
			 B.add_vertex_to_facet(2);
			 B.add_vertex_to_facet(1);
			 B.add_vertex_to_facet(0);
			 B.end_facet();*/

			 B.begin_facet();
			 B.add_vertex_to_facet(3);
			 B.add_vertex_to_facet(2);
			 B.add_vertex_to_facet(1);
			 B.end_facet();

			 B.begin_facet();
			 B.add_vertex_to_facet(1);
			 B.add_vertex_to_facet(0);
			 B.add_vertex_to_facet(3);
			 B.end_facet();

/*
			 B.begin_facet();
			 B.add_vertex_to_facet(4);
			 B.add_vertex_to_facet(5);
			 B.add_vertex_to_facet(6);
			 B.add_vertex_to_facet(7);
			 B.end_facet();*/

			 B.begin_facet();
			 B.add_vertex_to_facet(4);
			 B.add_vertex_to_facet(5);
			 B.add_vertex_to_facet(6);
			 B.end_facet();

			 B.begin_facet();
			 B.add_vertex_to_facet(6);
			 B.add_vertex_to_facet(7);
			 B.add_vertex_to_facet(4);
			 B.end_facet();

			 /*
			 B.begin_facet();
			 B.add_vertex_to_facet(4);
			 B.add_vertex_to_facet(0);
			 B.add_vertex_to_facet(1);
			 B.add_vertex_to_facet(5);
			 B.end_facet();*/

			 B.begin_facet();
			 B.add_vertex_to_facet(4);
			 B.add_vertex_to_facet(0);
			 B.add_vertex_to_facet(1);
			 B.end_facet();

			 B.begin_facet();
			 B.add_vertex_to_facet(1);
			 B.add_vertex_to_facet(5);
			 B.add_vertex_to_facet(4);
			 B.end_facet();

				/*
			 B.begin_facet();
			 B.add_vertex_to_facet(5);
			 B.add_vertex_to_facet(1);
			 B.add_vertex_to_facet(2);
			 B.add_vertex_to_facet(6);
			 B.end_facet();*/

			 B.begin_facet();
			 B.add_vertex_to_facet(5);
			 B.add_vertex_to_facet(1);
			 B.add_vertex_to_facet(2);
			 B.end_facet();

			 B.begin_facet();
			 B.add_vertex_to_facet(2);
			 B.add_vertex_to_facet(6);
			 B.add_vertex_to_facet(5);
			 B.end_facet();

			 /*
			 B.begin_facet();
			 B.add_vertex_to_facet(6);
			 B.add_vertex_to_facet(2);
			 B.add_vertex_to_facet(3);
			 B.add_vertex_to_facet(7);
			 B.end_facet();*/

			 B.begin_facet();
			 B.add_vertex_to_facet(6);
			 B.add_vertex_to_facet(2);
			 B.add_vertex_to_facet(3);
			 B.end_facet();

			 B.begin_facet();
			 B.add_vertex_to_facet(3);
			 B.add_vertex_to_facet(7);
			 B.add_vertex_to_facet(6);
			 B.end_facet();

			 /*
			 B.begin_facet();
			 B.add_vertex_to_facet(7);
			 B.add_vertex_to_facet(3);
			 B.add_vertex_to_facet(0);
			 B.add_vertex_to_facet(4);
			 B.end_facet();*/

			  B.begin_facet();
			 B.add_vertex_to_facet(7);
			 B.add_vertex_to_facet(3);
			 B.add_vertex_to_facet(0);
			 B.end_facet();

			 B.begin_facet();
			 B.add_vertex_to_facet(0);
			 B.add_vertex_to_facet(4);
			 B.add_vertex_to_facet(7);
			 B.end_facet();
			

		
        B.end_surface();
    }
};



template <class HDS>
class Build_circle : public CGAL::Modifier_base<HDS> {
private:
	double mradius;
	Point_d mpt;
	Vector_3 mnormal;
	bool mpureTriangle;
public:
    Build_circle(Point_d pt, Vector_3 normal, double radius, bool pureTriangle) {mradius = radius; mpt=pt;  mnormal=normal; mpureTriangle = pureTriangle;}
    void operator()( HDS& hds) {
        // Postcondition: `hds' is a valid polyhedral surface.
        CGAL::Polyhedron_incremental_builder_3<HDS> B(hds, true);

		const double step = 30; //degree

		int ratiostep = 360/step;
		int nb_point = ratiostep+1;	
		int nb_facet = ratiostep;	

		B.begin_surface( nb_point, nb_facet);			//nb point, nb facet
  
		Plane_3 plane(mpt,mnormal); 
		//Point_2 ptc=plane.to_2d(mpt); 
		std::vector<Point_d> list_pt1;

		for(int i=0;i<ratiostep;i++){
			Point_2 temp_2(cos(i*M_PI*step/180),sin(i*M_PI*step/180)); 
			Vector_3 B1=plane.base1(); FT norme1=sqrt(pow(B1.x(),2)+pow(B1.y(),2)+pow(B1.z(),2)); Vector_3 b1=(1/norme1)*B1;		
			Vector_3 B2=plane.base2(); FT norme2=sqrt(pow(B2.x(),2)+pow(B2.y(),2)+pow(B2.z(),2)); Vector_3 b2=(1/norme2)*B2;	
			Point_d temp_3(mpt.x()+mradius*(b1.x()*temp_2.x()+b2.x()*temp_2.y()),mpt.y()+mradius*(b1.y()*temp_2.x()+b2.y()*temp_2.y()),mpt.z()+mradius*(b1.z()*temp_2.x()+b2.z()*temp_2.y())); 

			list_pt1.push_back(temp_3);
		}


		//add point
		B.add_vertex(mpt);
		for(int i=0;i<ratiostep;i++) B.add_vertex(list_pt1[i]); 
	
		
		//add facet 
		for(int i=1;i<ratiostep;i++){
			 B.begin_facet();
			 B.add_vertex_to_facet(0);
			 B.add_vertex_to_facet(i);
			 B.add_vertex_to_facet(i+1);
			 B.end_facet();
		}
			 
			 B.begin_facet();
			 B.add_vertex_to_facet(0);
			 B.add_vertex_to_facet(ratiostep);
			 B.add_vertex_to_facet(1);
			 B.end_facet();

        B.end_surface();
    }
};



template <class HDS>
class Build_cone : public CGAL::Modifier_base<HDS> {
private:
	double mradius;
	double mheight;
	Point_d mpt;
	Vector_3 mnormal;
	bool mpureTriangle;
public:
    Build_cone(Point_d pt, Vector_3 normal, double radius, double height, bool pureTriangle) {mradius = radius; mheight=height; mpt=pt;  mnormal=normal; mpureTriangle = pureTriangle;}
    void operator()( HDS& hds) {
        // Postcondition: `hds' is a valid polyhedral surface.
        CGAL::Polyhedron_incremental_builder_3<HDS> B(hds, true);

		const double step = 5; //degree

		int ratiostep = 360/step;
		int nb_point = ratiostep+2;	
		int nb_facet = 2*ratiostep;	

		B.begin_surface( nb_point, nb_facet);			//nb point, nb facet
  
		double norm=sqrt(pow(mnormal.x(),2)+pow(mnormal.y(),2)+pow(mnormal.z(),2)); if(norm<1e-7) norm=1e-7;
		Vector_3 normal_normalise(mnormal.x()/norm,mnormal.y()/norm,mnormal.z()/norm);
		Point_d pth(mpt.x()+mheight*normal_normalise.x(),mpt.y()+mheight*normal_normalise.y(),mpt.z()+mheight*normal_normalise.z());

		Plane_3 plane(mpt,mnormal); 
		//Point_2 ptc=plane.to_2d(mpt); 
		std::vector<Point_d> list_pt1;

		for(int i=0;i<ratiostep;i++){
			Point_2 temp_2(cos(i*M_PI*step/180),sin(i*M_PI*step/180)); 
			Vector_3 B1=plane.base1(); FT norme1=sqrt(pow(B1.x(),2)+pow(B1.y(),2)+pow(B1.z(),2)); Vector_3 b1=(1/norme1)*B1;		
			Vector_3 B2=plane.base2(); FT norme2=sqrt(pow(B2.x(),2)+pow(B2.y(),2)+pow(B2.z(),2)); Vector_3 b2=(1/norme2)*B2;	
			Point_d temp_3(mpt.x()+mradius*(b1.x()*temp_2.x()+b2.x()*temp_2.y()),mpt.y()+mradius*(b1.y()*temp_2.x()+b2.y()*temp_2.y()),mpt.z()+mradius*(b1.z()*temp_2.x()+b2.z()*temp_2.y())); 

			list_pt1.push_back(temp_3);
		}


		//add point
		B.add_vertex(pth);
		B.add_vertex(mpt);
		for(int i=0;i<ratiostep;i++) B.add_vertex(list_pt1[i]); 
	
		
		//add facet 
		for(int i=2;i<ratiostep+1;i++){
			 B.begin_facet();
			 B.add_vertex_to_facet(0);
			 B.add_vertex_to_facet(i);
			 B.add_vertex_to_facet(i+1);
			 B.end_facet();
		}
			 
			 B.begin_facet();
			 B.add_vertex_to_facet(0);
			 B.add_vertex_to_facet(ratiostep+1);
			 B.add_vertex_to_facet(2);
			 B.end_facet();


		for(int i=2;i<ratiostep+1;i++){
			 B.begin_facet();
			 B.add_vertex_to_facet(i);
			 B.add_vertex_to_facet(1);
			 B.add_vertex_to_facet(i+1);
			 B.end_facet();
		}
			 
			 B.begin_facet();
			 B.add_vertex_to_facet(ratiostep+1);
			 B.add_vertex_to_facet(1);
			 B.add_vertex_to_facet(2);
			 B.end_facet();

        B.end_surface();
    }
};



inline Polyhedron createDisk(Point_d p, Vector_3 normal, double radius)
{
	Polyhedron P;
	Build_circle<HalfedgeDS> poly1(p,normal,radius,true);
	P.delegate(poly1);

	return P;
}

inline Polyhedron createCone(Point_d p, Vector_3 normal, double radius, double height)
{
	Polyhedron P;
	Build_cone<HalfedgeDS> poly1(p,normal,radius,height,true);
	P.delegate(poly1);

	return P;
}



inline Polyhedron createSquare(std::vector < Point_d > vp)
{
	Polyhedron P;
	Build_square<HalfedgeDS> poly1(vp,true);
	P.delegate(poly1);

	return P;
}


inline Polyhedron createCube(std::vector < Point_d > vp)
{
	Polyhedron P;
	Build_cube<HalfedgeDS> poly1(vp,true);
	P.delegate(poly1);

	return P;
}

// arrow from p1 pointing to p2
inline Polyhedron createArrow(Point_d p1, Point_d p2, double size)
{
	Polyhedron P;

	double pas=3.5;

	Vector_3 normal_temp(p2.x()-p1.x(),p2.y()-p1.y(),p2.z()-p1.z()); double norm=std::sqrt(normal_temp.squared_length()); if(norm<1e-7) norm = 1e-7;
	Vector_3 normal(normal_temp.x()/norm, normal_temp.y()/norm, normal_temp.z()/norm);
	Point_d pt1(p1.x()-normal.x()*pas*size,p1.y()-normal.y()*pas*size, p1.z()-normal.z()*pas*size);
	Point_d pt2(p2.x()+normal.x()*pas*size,p2.y()+normal.y()*pas*size, p2.z()+normal.z()*pas*size);

	Build_cylinder<HalfedgeDS> poly1(p1,pt2,size,true,30);
	P.delegate(poly1);

	// Build_cone<HalfedgeDS> poly2(pt1,-normal,pas/2*size,pas*size,true);
	// P.delegate(poly2);

	Build_cone<HalfedgeDS> poly3(pt2,normal,pas/2*size,pas*size,true);
	P.delegate(poly3);

	return P;
}






inline Polyhedron createDashedCylinder(Point_d p1, Point_d p2, double size)
{
	double pas=0.5/4;//0.03;
	double step=45;
	//double length=sqrt(pow(p1.x()-p2.x(),2)+pow(p1.y()-p2.y(),2)+pow(p1.z()-p2.z(),2));
	Vector_3 normal(p2,p1); //double norm=sqrt(pow(normal
	Point_d p1a(p1.x()+pas*normal.x(),p1.y()+pas*normal.y(),p1.z()+pas*normal.z());
	Point_d p2a(p1.x()+2*pas*normal.x(),p1.y()+2*pas*normal.y(),p1.z()+2*pas*normal.z());
	Point_d p3a(p1.x()+3*pas*normal.x(),p1.y()+3*pas*normal.y(),p1.z()+3*pas*normal.z());
	Point_d p4a(p1.x()+4*pas*normal.x(),p1.y()+4*pas*normal.y(),p1.z()+4*pas*normal.z());
	
	Point_d p1b(p2.x()-pas*normal.x(),p2.y()-pas*normal.y(),p2.z()-pas*normal.z());
	Point_d p2b(p2.x()-2*pas*normal.x(),p2.y()-2*pas*normal.y(),p2.z()-2*pas*normal.z());
	Point_d p3b(p2.x()-3*pas*normal.x(),p2.y()-3*pas*normal.y(),p2.z()-3*pas*normal.z());
	Point_d p4b(p2.x()-4*pas*normal.x(),p2.y()-4*pas*normal.y(),p2.z()-4*pas*normal.z());

	Polyhedron P;
	Build_cylinder<HalfedgeDS> poly1(p1,p2,size,true,step);
	P.delegate(poly1);
	
	Build_cylinder<HalfedgeDS> poly2(p1a,p2a,size,true,step);
	P.delegate(poly2);
	Build_cylinder<HalfedgeDS> poly3(p3a,p4a,size,true,step);
	P.delegate(poly3);
	Build_cylinder<HalfedgeDS> poly4(p1b,p2b,size,true,step);
	P.delegate(poly4);
	Build_cylinder<HalfedgeDS> poly5(p3b,p4b,size,true,step);
	P.delegate(poly5);

	Build_sphere<HalfedgeDS> poly1a(p1a,size,true,step);
	P.delegate(poly1a);
	Build_sphere<HalfedgeDS> poly2a(p2a,size,true,step);
	P.delegate(poly2a);
	Build_sphere<HalfedgeDS> poly3a(p3a,size,true,step);
	P.delegate(poly3a);
	Build_sphere<HalfedgeDS> poly4a(p4a,size,true,step);
	P.delegate(poly4a);

	Build_sphere<HalfedgeDS> poly1b(p1b,size,true,step);
	P.delegate(poly1b);
	Build_sphere<HalfedgeDS> poly2b(p2b,size,true,step);
	P.delegate(poly2b);
	Build_sphere<HalfedgeDS> poly3b(p3b,size,true,step);
	P.delegate(poly3b);
	Build_sphere<HalfedgeDS> poly4b(p4b,size,true,step);
	P.delegate(poly4b);


	return P;
}

inline Polyhedron createDashedCylinder2(Point_d p1, Point_d p2, double size)
{
	double pas=0.03;//0.03;
	double step=45;
	//double length=sqrt(pow(p1.x()-p2.x(),2)+pow(p1.y()-p2.y(),2)+pow(p1.z()-p2.z(),2));
	Vector_3 normal(p2,p1); //double norm=sqrt(pow(normal
	Point_d p1a(p1.x()-pas*normal.x(),p1.y()-pas*normal.y(),p1.z()-pas*normal.z());
	Point_d p2a(p1.x()-2*pas*normal.x(),p1.y()-2*pas*normal.y(),p1.z()-2*pas*normal.z());
	Point_d p3a(p1.x()-3*pas*normal.x(),p1.y()-3*pas*normal.y(),p1.z()-3*pas*normal.z());
	Point_d p4a(p1.x()-4*pas*normal.x(),p1.y()-4*pas*normal.y(),p1.z()-4*pas*normal.z());
	
	Point_d p1b(p2.x()+pas*normal.x(),p2.y()+pas*normal.y(),p2.z()+pas*normal.z());
	Point_d p2b(p2.x()+2*pas*normal.x(),p2.y()+2*pas*normal.y(),p2.z()+2*pas*normal.z());
	Point_d p3b(p2.x()+3*pas*normal.x(),p2.y()+3*pas*normal.y(),p2.z()+3*pas*normal.z());
	Point_d p4b(p2.x()+4*pas*normal.x(),p2.y()+4*pas*normal.y(),p2.z()+4*pas*normal.z());

	Polyhedron P;
	Build_cylinder<HalfedgeDS> poly1(p4a,p4b,size,true,step);
	P.delegate(poly1);
	
	Build_cylinder<HalfedgeDS> poly2(p2a,p3a,size,true,step);
	P.delegate(poly2);
	Build_cylinder<HalfedgeDS> poly3(p1,p1a,size,true,step);
	P.delegate(poly3);
	Build_cylinder<HalfedgeDS> poly4(p2b,p3b,size,true,step);
	P.delegate(poly4);
	Build_cylinder<HalfedgeDS> poly5(p2,p1b,size,true,step);
	P.delegate(poly5);

	Build_sphere<HalfedgeDS> poly1a(p1a,size,true,step);
	P.delegate(poly1a);
	Build_sphere<HalfedgeDS> poly2a(p2a,size,true,step);
	P.delegate(poly2a);
	Build_sphere<HalfedgeDS> poly3a(p3a,size,true,step);
	P.delegate(poly3a);
	Build_sphere<HalfedgeDS> poly4a(p4a,size,true,step);
	P.delegate(poly4a);

	Build_sphere<HalfedgeDS> poly1b(p1b,size,true,step);
	P.delegate(poly1b);
	Build_sphere<HalfedgeDS> poly2b(p2b,size,true,step);
	P.delegate(poly2b);
	Build_sphere<HalfedgeDS> poly3b(p3b,size,true,step);
	P.delegate(poly3b);
	Build_sphere<HalfedgeDS> poly4b(p4b,size,true,step);
	P.delegate(poly4b);


	return P;
}


inline Polyhedron createSphere(Point_d p, double size)
{
	Polyhedron P;
	double step=60;
	Build_sphere<HalfedgeDS> poly1(p,size,true,step);
	P.delegate(poly1);

	return P;
}

inline Polyhedron createSphere_fine(Point_d p, double size)
{
	Polyhedron P;
	double step=30;
	Build_sphere<HalfedgeDS> poly1(p,size,true,step);
	P.delegate(poly1);

	return P;
}

inline Polyhedron createCylinder(Point_d p1, Point_d p2, double size)
{
	Polyhedron P;
	double step=90;
	Build_cylinder<HalfedgeDS> poly1(p1,p2,size,true,step);
	P.delegate(poly1);

	return P;
}

inline Polyhedron createCylinder_fine(Point_d p1, Point_d p2, double size)
{
	Polyhedron P;
	double step=30;
	Build_cylinder<HalfedgeDS> poly1(p1,p2,size,true,step);
	P.delegate(poly1);

	return P;
}



inline bool save_listpolyhedron2ply(std::vector<Polyhedron> lP, std::string& nom, std::vector<CGAL::Color> lC)
{
	std::cout <<"Saving polyhedron to "<<nom<<std::endl;
	OFSTREAM_TEXTE(fic,nom);
	
	int nb_vertices = 0;
	int nb_facets = 0;

	std::vector<Polyhedron>::iterator it;
	for(it = lP.begin(); it != lP.end(); it++)
	{				
		nb_vertices+=(*it).size_of_vertices ();
		nb_facets+=(*it).size_of_facets ();
	}

	//header
	fic<<"ply"<<std::endl;
	fic<<"format ascii 1.0"<<std::endl;
	fic<<"element vertex "<<nb_vertices<<std::endl;
	fic<<"property float x"<<std::endl;
	fic<<"property float y"<<std::endl;
	fic<<"property float z"<<std::endl;
	fic<<"element face "<<nb_facets<<std::endl;
	fic<<"property list uchar int vertex_index"<<std::endl;
	fic<<"property uchar red"<<std::endl;
	fic<<"property uchar green"<<std::endl;
	fic<<"property uchar blue"<<std::endl;
	fic<<"end_header"<<std::endl;

	//vertex list
	std::map<Point_d,int> map_point_index;
	int count = 0;
	for(it = lP.begin(); it != lP.end(); it++)
	{			
		for( Polyhedron::Vertex_const_iterator vi = (*it).vertices_begin(); vi != (*it).vertices_end(); ++vi) 
		{
			map_point_index[vi->point()] = count++;
			fic<<vi->point().x()<<" "<<vi->point().y()<<" "<<vi->point().z()<<std::endl;
		}
	}

	int i=0;
	for(it = lP.begin(); it != lP.end(); it++, i++)
	{	
		//facet list
		Polyhedron::Halfedge_around_facet_const_circulator hc, hc_end;
		for(  Polyhedron::Facet_const_iterator fi = (*it).facets_begin(); fi != (*it).facets_end(); ++fi)
		{
			hc = fi->facet_begin();
			hc_end = hc;
			std::size_t n = circulator_size( hc);
			CGAL_assertion( n >= 3);
			fic<<n<<" ";
			do {
				fic<<map_point_index[hc->vertex()->point()]<<" ";
				++hc;
			} while( hc != hc_end);
			fic<<(int)lC[i].r()<<" "<<(int)lC[i].g()<<" "<<(int)lC[i].b()<<std::endl;
		}
	}
		
fic<<std::endl;	

return true;
}




inline bool colorplanset2ply( std::string& filename, const std::vector < std::vector < Point_d > > & pt, const std::vector< CGAL::Color > & colors)
	{

	std::cout <<"Saving planes to "<<filename<<std::endl;
	OFSTREAM_TEXTE(fic,filename);
	
	int nb_pts=0;
	for(size_t i=0;i<pt.size();i++){nb_pts=nb_pts+pt[i].size();}
		

	//header
		fic<<"ply"<<std::endl;
		fic<<"format ascii 1.0"<<std::endl;
		fic<<"comment author: F. Lafarge"<<std::endl;
		fic<<"element vertex "<<nb_pts<<std::endl;
		fic<<"property float x"<<std::endl;
		fic<<"property float y"<<std::endl;
		fic<<"property float z"<<std::endl;
		fic<<"property uchar red"<<std::endl;
		fic<<"property uchar green"<<std::endl;
		fic<<"property uchar blue"<<std::endl;
		fic<<"element face "<<pt.size()<<std::endl;
		fic<<"property list uchar int vertex_index"<<std::endl;
		fic<<"end_header"<<std::endl;

		//vertex list

		for(size_t i=0;i<pt.size();i++){
			for(size_t j=0;j<pt[i].size();j++){
				fic<<pt[i][j].x()<<" "<<pt[i][j].y()<<" "<<pt[i][j].z()<<" "<<(int)colors[i].r()<<" "<<(int)colors[i].g()<<" "<<(int)colors[i].b()<<std::endl;
			}
		}
		int cont=0;
		for(size_t i=0;i<pt.size();i++){
			fic<<pt[i].size()<<" ";
			for(size_t k=cont;k<cont+pt[i].size();k++){
				fic<<k<<" ";
			}
			fic<<std::endl;
			cont=cont+pt[i].size();
		}

fic<<std::endl;	

return true;
	}



inline bool color_inliers( std::string& filename, std::vector < std::vector < Point_d > > & pt, std::vector< CGAL::Color > & colors)
	{

	std::cout <<"Saving planes to "<<filename<<std::endl;
	OFSTREAM_TEXTE(fic,filename);
	
	size_t nb_pts=0;
	for(size_t i=0;i<pt.size();i++){nb_pts=nb_pts+pt[i].size();}
		

	//header
		fic<<"ply"<<std::endl;
		fic<<"format ascii 1.0"<<std::endl;
		fic<<"comment author: F. Lafarge"<<std::endl;
		fic<<"element vertex "<<nb_pts<<std::endl;
		fic<<"property float x"<<std::endl;
		fic<<"property float y"<<std::endl;
		fic<<"property float z"<<std::endl;
		fic<<"property uchar red"<<std::endl;
		fic<<"property uchar green"<<std::endl;
		fic<<"property uchar blue"<<std::endl;
		fic<<"end_header"<<std::endl;

		//vertex list

		for(size_t i=0;i<pt.size();i++){
			for(size_t j=0;j<pt[i].size();j++){
				fic<<pt[i][j].x()<<" "<<pt[i][j].y()<<" "<<pt[i][j].z()<<" "<<(int)colors[i].r()<<" "<<(int)colors[i].g()<<" "<<(int)colors[i].b()<<std::endl;
			}
		}

fic<<std::endl;	

return true;
	}



inline double bbox_diag(Bbox& bbox)
{
	double dx = bbox.xmax() - bbox.xmin();
	double dy = bbox.ymax() - bbox.ymin();
	double dz = bbox.zmax() - bbox.zmin();
	return std::sqrt(dx*dx + dy*dy + dz*dz);
}


template <class Cloud_segmentation>
inline bool save_envelops_with_normal(Cloud_segmentation C, std::string output_folder = ".")
{
	std::vector < std::vector < Point_d > > point3dres_graham;
	std::vector < std::vector < Point_d > > point3dres_alpha_shape;
	std::vector<std::pair<Vector , double>> plane_normals;

	std::vector < CGAL::Color > color_col_graham;
	std::vector < CGAL::Color > color_col_alpha_shape;
	std::vector < CGAL::Color > color_points_normal;

	int val_lab=0;
	int counting1=0;
	C.extracted_planes.clear();

	std::vector<CGAL::Color> vector_of_colors;
	for(int i=0; i<(int)C.plane_point_index.size();i++){
		int red=(int)floor((double)126*rand()/RAND_MAX)+130;
		int green=(int)floor((double)126*rand()/ RAND_MAX)+130;
		int blue=(int)floor((double)126*rand()/ RAND_MAX)+130;
		CGAL::Color col(std::max(0,std::min(255,red)),std::max(0,std::min(255,green)),std::max(0,std::min(255,blue)),120);
		vector_of_colors.push_back(col);
	}
	
	std::vector < std::vector < Point_d > > listpts;

	for(int i=0; i<(int)C.plane_point_index.size();i++){
		
        std::vector < Point_d > listp;
        std::vector < Vector > listn;
        for(int k=0; k<(int)C.plane_point_index[i].size(); k++){
            int yy=C.plane_point_index[i][k];
            Point_d pt=C.HPS[yy].position;
            Vector n=C.HPS[yy].normal;
            listp.push_back(pt);
            listn.push_back(n);
        }
		listpts.push_back(listp);

		val_lab++;
		Plane_3 plane;
		double erro=linear_least_squares_fitting_3(listp.begin(),listp.end(),plane, CGAL::Dimension_tag<0>());
		C.extracted_planes.push_back(plane);

        double normal_dir = 0;
        for (int k=0; k<(int)listn.size(); ++k) {
            normal_dir += listn[k]*plane.orthogonal_vector();
        }
        Vector normal_vector = normal_dir > 0 ? plane.orthogonal_vector() : -plane.orthogonal_vector();
		normal_vector /= sqrt(normal_vector.squared_length());
		plane_normals.push_back( std::pair<Vector , double>(normal_vector, double(listp.size())) );
		
		CGAL::Color col=vector_of_colors[i];
		color_points_normal.push_back(col);

		Vector_3 vortho=plane.orthogonal_vector();
		Vector_3 b1=plane.base1();
		Vector_3 b2=plane.base2();
		FT norme1=sqrt(pow(b1.x(),2)+pow(b1.y(),2)+pow(b1.z(),2)); if(norme1<1e-7){norme1=1e-7;}
		FT norme2=sqrt(pow(b2.x(),2)+pow(b2.y(),2)+pow(b2.z(),2)); if(norme2<1e-7){norme2=1e-7;}

		Vector_3 vb1=(1/norme1)*b1;
		Vector_3 vb2=(1/norme2)*b2;

		std::vector< Point_d > poinn;
		std::vector< Point_2d > poin2;

        //Point_d pto=plane.projection(Point_d(0.,0.,0.));
        for(int k=0; k<(int)C.plane_point_index[i].size();k++){
            int ind=C.plane_point_index[i][k];
            Point_d pt=C.HPS[ind].position;
            Point_d ptp=plane.projection(pt);
            FT X1=vb1.x()*ptp.x()+vb1.y()*ptp.y()+vb1.z()*ptp.z();
            FT X2=vb2.x()*ptp.x()+vb2.y()*ptp.y()+vb2.z()*ptp.z();
            Point_2d ptp2(X1,X2);
            poinn.push_back(ptp);
            poin2.push_back(ptp2);
        }

		std::vector < Point_2d > poin2res_graham;
		CGAL::ch_graham_andrew(poin2.begin(),poin2.end(),std::back_inserter(poin2res_graham));
		Alpha_Shape AlphaShape(poin2.begin(),poin2.end(),FT(std::pow(C.alpha_thresh*C.average_k_spacing,2)),Alpha_Shape::REGULARIZED);   
		
		std::vector < std::pair < Point_2d, Point_2d> > pairs_edges;
		std::vector < bool > active_pairs;
		for (Alpha_Shape::Alpha_shape_edges_iterator it = AlphaShape.alpha_shape_edges_begin(); it != AlphaShape.alpha_shape_edges_end(); ++it){
			Alpha_Shape::Edge edg= *it;
			Point_2d pe1=edg.first->vertex(edg.first->ccw(edg.second))->point();
			Point_2d pe2=edg.first->vertex(edg.first->cw(edg.second))->point();
		
			std::pair < Point_2d, Point_2d> pair_ed;
			pair_ed.first=pe1;
			pair_ed.second=pe2;
			pairs_edges.push_back(pair_ed);
			active_pairs.push_back(true);
		}

		std::vector < Point_2d > ranked_pts;
		if(pairs_edges.size()>0){
			ranked_pts.push_back(pairs_edges[0].first); 
			ranked_pts.push_back(pairs_edges[0].second); 
			active_pairs[0]=false;
			Point_2d pt_to_find=pairs_edges[0].second;
			do{
				for(int iter=1;iter<pairs_edges.size();iter++){
					if(active_pairs[iter]){
						if(pt_to_find == pairs_edges[iter].first) { ranked_pts.push_back( pairs_edges[iter].second);  pt_to_find=pairs_edges[iter].second; active_pairs[iter]=false; }
						else if(pt_to_find == pairs_edges[iter].second) { ranked_pts.push_back( pairs_edges[iter].first); pt_to_find=pairs_edges[iter].first; active_pairs[iter]=false; }
					}
				}
			}while(pt_to_find!=pairs_edges[0].first);
		}

		Polygon_2 poly2(ranked_pts.begin(),ranked_pts.end());
		CDT2 cdt;

		for(Polygon_2::Vertex_iterator it2 = poly2.vertices_begin () ; it2 != poly2.vertices_end ();++it2) cdt.insert(*it2);

		CGAL::make_conforming_Delaunay_2(cdt);

		std::vector < Polygon_2 >  partition_polys;
		for ( CDT2::Finite_faces_iterator  vit = cdt.finite_faces_begin(); vit != cdt.finite_faces_end(); ++vit){        
				vector<Point_2d> p(3);
				p[0] = vit->vertex (0)->point();
				p[1] = vit->vertex (1)->point();
				p[2] = vit->vertex (2)->point();
				Point_2d bary( (p[0].x()+ p[1].x() + p[2].x())/3.,(p[0].y()+ p[1].y() + p[2].y())/3.);

				if (CGAL::bounded_side_2(poly2.vertices_begin (), poly2.vertices_end (),bary, Kernel()) != CGAL::ON_UNBOUNDED_SIDE)
				{
					Polygon_2 po(p.begin(),p.end());
					partition_polys.push_back(po);
                    vector<Point_d> pd(3);
                    FT X1=p[0].x()*vb1.x()+p[0].y()*vb2.x()-plane.d()*vortho.x();
                    FT Y1=p[0].x()*vb1.y()+p[0].y()*vb2.y()-plane.d()*vortho.y();
                    FT Z1=p[0].x()*vb1.z()+p[0].y()*vb2.z()-plane.d()*vortho.z();
                    pd[0] = Point_d(X1,Y1,Z1); 
                    FT X2=p[1].x()*vb1.x()+p[1].y()*vb2.x()-plane.d()*vortho.x();
                    FT Y2=p[1].x()*vb1.y()+p[1].y()*vb2.y()-plane.d()*vortho.y();
                    FT Z2=p[1].x()*vb1.z()+p[1].y()*vb2.z()-plane.d()*vortho.z();
                    pd[1] = Point_d(X2,Y2,Z2); 
                    FT X3=p[2].x()*vb1.x()+p[2].y()*vb2.x()-plane.d()*vortho.x();
                    FT Y3=p[2].x()*vb1.y()+p[2].y()*vb2.y()-plane.d()*vortho.y();
                    FT Z3=p[2].x()*vb1.z()+p[2].y()*vb2.z()-plane.d()*vortho.z();
                    pd[2] = Point_d(X3,Y3,Z3); 
					point3dres_alpha_shape.push_back(pd);
					color_col_alpha_shape.push_back(col);
				}
		}


		std::vector < Point_d > poin3res;
		for(int k=0; k<(int)poin2res_graham.size();k++){
			FT X1=poin2res_graham[k].x()*vb1.x()+poin2res_graham[k].y()*vb2.x()-plane.d()*vortho.x();
			FT X2=poin2res_graham[k].x()*vb1.y()+poin2res_graham[k].y()*vb2.y()-plane.d()*vortho.y();
			FT X3=poin2res_graham[k].x()*vb1.z()+poin2res_graham[k].y()*vb2.z()-plane.d()*vortho.z();
			Point_d tempo(X1,X2,X3);
			poin3res.push_back(tempo);
		}
		point3dres_graham.push_back(poin3res);
		color_col_graham.push_back(col);

	} 

	std::string sep(1, boost::filesystem::path::preferred_separator);
	std::string outdir = output_folder + sep + "shape_detection";
	boost::filesystem::create_directory(outdir);
	// std::string namo_inliers_colored= outdir + sep + "Results_inliers.ply";
	// color_inliers(namo_inliers_colored,listpts,vector_of_colors);

	std::string namo_graham= outdir + sep + "Results_convex_envelops.ply";
	colorplanset2ply(namo_graham,point3dres_graham,color_col_graham);

	std::string namo_alphashape= outdir + sep + "Results_alpha_shapes.ply";
	colorplanset2ply(namo_alphashape,point3dres_alpha_shape,color_col_alpha_shape);

	// Visualize plane normals
	Vector tempc(0., 0., 0.);
	double x_min, y_min, z_min, x_max, y_max, z_max;
	x_min = y_min = z_min = FLT_MAX;
	x_max = y_max = z_max = -FLT_MAX;
	for (size_t i = 0 ; i < C.HPS.size() ; ++i) {
		const Point3 & p = C.HPS[i].position;
		tempc += Vector(p.x(), p.y(), p.z());
		const double &x = p.x(), &y = p.y(), &z = p.z();
		if (x < x_min) x_min = x;
		if (x > x_max) x_max = x;
		if (y < y_min) y_min = y;
		if (y > y_max) y_max = y;
		if (z < z_min) z_min = z;
		if (z > z_max) z_max = z;
	}
	Point3 pos_O(tempc.x()/C.HPS.size(), tempc.y()/C.HPS.size(), tempc.z()/C.HPS.size());
	Bbox points_bbox(x_min, y_min, z_min, x_max, y_max, z_max);
	double axis_length = bbox_diag(points_bbox) * 0.3;
	std::vector<Polyhedron> lP_normal_vectors;
	for (auto normal_weight : plane_normals) {
		Vector dir = normal_weight.first;
		Point3 pos_A = pos_O + axis_length*dir;
		lP_normal_vectors.push_back(createArrow(pos_O, pos_A, axis_length*0.03));
	}
	lP_normal_vectors.push_back(createSphere_fine(pos_O, axis_length*0.03));
	color_points_normal.push_back(CGAL::Color(210,210,210,160));
	std::string namo_points_normals = outdir + sep + "Results_normals.ply";
	save_listpolyhedron2ply(lP_normal_vectors, namo_points_normals, color_points_normal);

	return true;
}


}
#endif
