#ifndef _TYPES_
#define _TYPES_

// #include <CGAL/Simple_cartesian.h>
#include <CGAL/Exact_predicates_inexact_constructions_kernel.h>
#include <CGAL/Surface_mesh.h>
#include <CGAL/Polyhedron_3.h>
#include <CGAL/bounding_box.h>
#include <CGAL/Search_traits_3.h>
#include <CGAL/Orthogonal_k_neighbor_search.h>
#include <CGAL/Min_sphere_of_points_d_traits_3.h>
#include <CGAL/Min_sphere_of_spheres_d.h>
#include <list>
#include <vector>

typedef enum {LineSearch, TrustRegion} Solver_Type;

const char separator =
#ifdef _WIN32
    '\\';
#else
    '/';
#endif

#ifndef CGAL_HAS_THREADS
#define CGAL_HAS_THREADS
#endif

// kernel
typedef CGAL::Exact_predicates_inexact_constructions_kernel Kernel;
// typedef CGAL::Simple_cartesian<double> Kernel;
// basic types 
typedef Kernel::FT FT;
typedef CGAL::Bbox_3 Bbox;
typedef Kernel::Ray_3 Ray;
typedef Kernel::Line_3 Line;
typedef Kernel::Point_3 Point3;
typedef Kernel::Plane_3 Plane;
typedef Kernel::Vector_3 Vector;
typedef Kernel::Sphere_3 Sphere;
typedef Kernel::Segment_3 Segment;
typedef Kernel::Triangle_3 Triangle;
typedef Kernel::Aff_transformation_3 Aff_transformation;

typedef std::pair<Point3,FT>               Point_distance;
typedef std::pair<Point3,Vector>           Point_with_normal;

// point list
typedef std::list<Point3> PointList;
typedef std::vector<Point_with_normal>     Pwn_vector;

typedef typename Pwn_vector::value_type PointRange_t;

// segment list
typedef std::list<Segment> SegmentList;
// triangle list
typedef std::list<Triangle> TriangleList;

typedef CGAL::First_of_pair_property_map<Point_with_normal>  Point_map;
typedef CGAL::Second_of_pair_property_map<Point_with_normal> Normal_map;

// surface mesh data structure
typedef CGAL::Surface_mesh<Point3> Mesh;
typedef Mesh::Vertex_index         Vertex_index;
typedef Mesh::Face_index           Face_index;
typedef boost::graph_traits<Mesh>::face_descriptor                   face_descriptor;
typedef boost::graph_traits<Mesh>::vertex_descriptor                 vertex_descriptor;
typedef boost::graph_traits<Mesh>::halfedge_descriptor               halfedge_descriptor;

typedef CGAL::Polyhedron_3<Kernel>      Polyhedron;

// KNN search
typedef CGAL::Search_traits_3<Kernel> TreeTraits;
typedef CGAL::Orthogonal_k_neighbor_search<TreeTraits> Neighbor_search;
typedef Neighbor_search::Tree Tree;

// Min sphere
typedef CGAL::Min_sphere_of_points_d_traits_3<Kernel,double> MinSphereTraits;
typedef CGAL::Min_sphere_of_spheres_d<MinSphereTraits>    Min_sphere;

#endif // _TYPES_
