#ifndef _UTILS_
#define _UTILS_

#include <CGAL/Timer.h>
#include <boost/function.hpp>
#include <boost/bind.hpp>
#include "types.h"
#include <vector>

inline boost::function<Point3(PointRange_t&)> pwn_it_to_point_it = boost::bind(&PointRange_t::first, _1);
inline boost::function<Vector(PointRange_t&)> pwn_it_to_normal_it = boost::bind(&PointRange_t::second, _1);

struct Timeout_callback
{
  mutable int nb;
  mutable CGAL::Timer timer;
  const double limit;
  
  Timeout_callback(double limit) : nb(0), limit(limit)
  {
    timer.start();
  }
  
  bool operator()(double advancement) const
  {
    // Avoid calling time() at every single iteration, which could
    // impact performances very badly
    ++ nb;
    if (nb % 1000 != 0)
      return true;
    // If the limit is reach, interrupt the algorithm
    if (timer.time() > limit)
    {
      std::cerr << "Algorithm takes too long, exiting ("
                << 100. * advancement << "% done)" << std::endl;
      return false;
    }
    return true;
  }
};

template<class T1, class T2>
inline void extract_first(const std::vector<std::pair<T1, T2> >& v, std::vector<T1>& v1) {
    v1.resize(v.size());
    for (size_t i = 0; i < v.size(); i++) {
        v1[i] = v[i].first;
    }
}

template<class T1, class T2>
inline void extract_second(const std::vector<std::pair<T1, T2> >& v, std::vector<T2>& v2) {
    v2.resize(v.size());
    for (size_t i = 0; i < v.size(); i++) {
        v2[i] = v[i].second;
    }
}

#endif // _UTILS_