#ifndef _VIEWER_
#define _VIEWER_

#include <CGAL/Qt/qglviewer.h>
#include <QOpenGLVertexArrayObject>
#include <QOpenGLBuffer>
#include <QOpenGLShaderProgram>
#include "types.h"

class QWidget;
class Scene;


class Viewer : public CGAL::QGLViewer {
    Q_OBJECT

private:
	typedef Point3                    Point;
private:
    Scene* m_pScene;
	bool m_custom_mouse;
    QOpenGLShaderProgram m_shader;
	QOpenGLShaderProgram m_tex_shader;
    QOpenGLVertexArrayObject m_vao[7];
    QOpenGLBuffer m_vbo[14];
	bool are_buffers_initialized;

	GLuint textureId;

	// rendering options
	bool m_view_input_points;
	bool m_view_input_poyhedron;
	bool m_view_plane_sdf;
	bool m_view_plane_gradient;
	bool m_view_extracted_shapes;
	
public:
    Viewer(QWidget * parent);
	virtual ~Viewer();
	void setScene(Scene* pScene) { this->m_pScene = pScene; }
	void toggle_view_input_points()  { m_view_input_points = !m_view_input_points; } 
	void toggle_view_input_polyhedron()  { m_view_input_poyhedron = !m_view_input_poyhedron; } 
	void toggle_view_plane_sdf()  { m_view_plane_sdf = !m_view_plane_sdf; } 
	void toggle_view_plane_gradient()  { m_view_plane_gradient = !m_view_plane_gradient; } 
	void toggle_view_extracted_shapes() { m_view_extracted_shapes = !m_view_extracted_shapes; }
	
	// cutting plane activation/deactivation
	void activate_cutting_plane();
	void deactivate_cutting_plane();
	
	void initializeGL();
	void update_input_buffers();
	void draw();
	void update_bbox();

public slots:
	void cutting_plane(bool override = false);
	void changed();

private:
	Plane frame_plane() const;
	Aff_transformation frame_transformation() const;

protected:
    virtual void mousePressEvent(QMouseEvent* e);
    virtual void mouseReleaseEvent(QMouseEvent* e);

};

#endif // _VIEWER_
